/*
 * Copyright (C) 2012 Casey Schaufler <casey@schaufler-ca.com>
 * Copyright (C) Intel Corporation.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <pwd.h>
#include "smacktools.h"

int
main(int argc, char *argv[])
{
	int i;
	int fd;
	int smacklen;
	int violations = 0;
	int newargc;
	char *newargv[256]; /* yes, I know */
	char *selfish1 = "/proc/self/attr/smack/current";
	char *selfish2 = "/proc/self/attr/smack.current";
	char *selfish3 = "/proc/self/attr/current";
	char *smack;
	struct passwd *pwdp;

	if (argc <= 1) {
		fprintf(stderr, "%s: No value specified\n", argv[0]);
		exit(1);
	}
	/*
	 * Arguments for newsmack itself
	 */
	for (i = 1; i < argc && *argv[i] == '-'; i++) {
		if (strcmp(argv[i], "-v") == 0)
			violations = 1;
		else if (strcmp(argv[i], "-violations") == 0)
			violations = 1;
		else {
			fprintf(stderr,
				"%s: Unknown option \"%s\" specified\n",
				argv[0], argv[i]);
			exit(1);
		}
	}

	/*
	 * There must be another argument, the label
	 */
	if (i == argc) {
		fprintf(stderr, "%s: No value specified\n", argv[0]);
		exit(1);
	}

	smack = argv[i++];

	if ((smacklen = strlen(smack)) >= SMACK_LABEL || smacklen < 1) {
		fprintf(stderr, "%s: Bad value \"%s\" specified\n",
			argv[0], smack);
		exit(1);
	}

	pwdp = getpwuid(getuid());
	if (pwdp == NULL) {
		fprintf(stderr, "%s: User \"%s\" not known\n",
			argv[0], pwdp->pw_name);
		exit(1);
	}
	if (smack_user_allowed(pwdp->pw_name, smack) != 0) {
		fprintf(stderr, "%s: User \"%s\" not allowed smack \"%s\"\n",
			argv[0], pwdp->pw_name, smack);
		exit(1);
	}
	/*
	 * Start a shell if no command is specified
	 */

	if (i == argc) {
		fprintf(stderr, "%s: start a shell at \"%s\"\n",
			argv[0], smack);
		newargv[0] = "sh";
		newargv[1] = NULL;
	}
	else {
		for (newargc = 0; i < argc; newargc++, i++) {
			newargv[newargc] = argv[i];
		}
		newargv[newargc] = NULL;
	}


	/*
	 * If violations are set do everything to make it
	 * "work" anyway, that is, it's okay to perform
	 * acts that violate rational policies.
	 */
	if (violations) {
		if (setxattr("/proc/self/fd/0","security.SMACK64","*",1,0) < 0)
			perror("set smack attr");
	}
	/*
	 * Set the process label.
	 */
	if ((fd = open(selfish1, O_RDWR)) < 0 &&
	    (fd = open(selfish2, O_RDWR)) < 0 &&
	    (fd = open(selfish3, O_RDWR)) < 0) {
		fprintf(stderr, "%s: cannot open to set smack\n", argv[0]);
		exit(1);
	}
	if (write(fd, smack, smacklen) < 0) {
		fprintf(stderr, "%s: cannot write to set smack\n", argv[0]);
		exit(1);
	}
	close(fd);
	/*
	 * Do the exec
	 */
	execvp(newargv[0], newargv);

	fprintf(stderr, "%s: exec failure.\n", argv[0]);
	exit(0);
}
