/*
 * Copyright (C) 2012 Casey Schaufler <casey@schaufler-ca.com>
 * Copyright (C) Intel Corporation.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <linux/ip.h>
#include <linux/udp.h>
#include <linux/netlink.h>
#include <linux/xattr.h>

#include "smacktools.h"

#define BUFFER_SIZE 500

#define LOCAL_PORT	7000

int
main(int argc, char *argv[])
{
	int sock;
	struct sockaddr_in sin;
	unsigned short local_port = LOCAL_PORT;
	struct iphdr *ip_header;
	struct udphdr *udp_header;
	int tmp;
	int len;
	int i;
	int oadd;
	unsigned char *oap = (unsigned char *)&oadd;
	char peer[SMACK_LABEL];
	char cpeer[SMACK_LABEL];
	char rpeer[SMACK_LABEL];
	char callno[SMACK_LABEL];
	char from[SMACK_LABEL];
	char *ipin = NULL;
	int quiet = 0;
	int special;
	int right = 1;
	int isone = 1;
	char buffer[BUFFER_SIZE];
	char control[BUFFER_SIZE];
	struct iovec iov = { buffer, sizeof(buffer) };
	struct msghdr message = {
		(void*)&sin, sizeof(sin), &iov, 1, control, BUFFER_SIZE, 0
	};

	for (i = 1; i < argc; i++) {
		if (strcmp(argv[i], "-i") == 0 ||
		    strcmp(argv[i], "--ipin") == 0)
			ipin = argv[++i];
		if (strcmp(argv[i], "-p") == 0 ||
		    strcmp(argv[i], "--port") == 0)
			local_port = atoi(argv[++i]);
		if (strcmp(argv[i], "-q") == 0 ||
		    strcmp(argv[i], "--quiet") == 0)
			quiet = 1;
	}

	if ((sock = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0) { 
		perror("socket");
		exit(1);
	}

	bzero((char *)&sin, sizeof(sin));
	sin.sin_family = AF_INET;
	sin.sin_port = htons(local_port);

	if (bind(sock, (struct sockaddr *)& sin, sizeof(sin)) < 0) {
		perror("bind");
		exit(1);
	}

        if (setsockopt(sock, SOL_IP, IP_PASSSEC, &isone, sizeof(isone)) < 0) {
		perror("setsockopt IP_PASSEC");
		exit(1);
	}
  
	if (ipin != NULL) {
		tmp = fsetxattr(sock, "security.SMACK64IPIN",
				ipin, strlen(ipin)+1, 0);
		if (tmp < 0)
			perror("fsetxattr");
		else
			printf("security.SMACK64IPIN set = %d\n", tmp);
	}

	while ( 1 ) {
		len = sizeof(sin);
		bzero((char *)&sin, sizeof(sin));
		bzero(buffer, BUFFER_SIZE);
		bzero(control, BUFFER_SIZE);
		special = 0;

		message.msg_namelen = sizeof(sin);
		message.msg_controllen = BUFFER_SIZE;

		strcpy(peer, "UNAVAILABLE");
		i = smackrecvmsg(sock, &message, 0, peer, sizeof(peer));
		if (i < 0)
			fprintf(stderr, "%s:%d smackrecvmsg failed\n",
				__func__, __LINE__);
		i = smack_get_peer(sock, cpeer, rpeer);
		if (i < 0) {
			strcpy(cpeer, "<>");
			strcpy(rpeer, "<>");
		}

		i = strlen(peer);
		if (strncmp(buffer, peer, i) != 0)
			special = 1;
		else if (buffer[i] != ' ' && buffer[i] != '\0')
			special = 1;

		if (quiet == 0 || special != 0) {
			oadd = sin.sin_addr.s_addr;
			printf("From %u.%u.%u.%u port %d %d \"%s\" "
			       "peer = \"%s\" (\"%s\" \"%s\")"
			       "right %d\n",
				oap[0], oap[1], oap[2], oap[3],
				ntohs(sin.sin_port), i, buffer, peer,
				cpeer, rpeer, right);
		}
		if (special != 0)
			right = 0;
		else
			right++;
	}

	close(sock);

	return 0;
}
