/*
 * Copyright (C) 2007 Casey Schaufler <casey@schaufler-ca.com>
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation, version 2.
 *
 * Authors:
 *	Casey Schaufler <casey@schaufler-ca.com>
 *	Ahmed S. Darwish <darwish.07@gmail.com>
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include "smacktools.h"

int smackrecvmsg(int sock, struct msghdr *msgp, int flags,
		 char *smack, int smacklen)
{
	struct cmsghdr *chp;
	char *cp;
	int rlen;
	int len;
	int rc;

	rc = recvmsg(sock, msgp, flags);
	if (rc < 0) {
		perror("recvmsg error");
		return -1;
	}
	if (msgp->msg_controllen <= sizeof(struct cmsghdr)) {
		fprintf(stderr, "%s out of control %d <= %d\n", __func__,
			msgp->msg_controllen, sizeof(struct cmsghdr));
		return -1;
	}
	chp = CMSG_FIRSTHDR(msgp);
	if (chp->cmsg_type != SCM_SECURITY) {
		fprintf(stderr, "%s type not SCM_SECURITY\n", __func__);
		return -1;
	}
	cp = (char *) CMSG_DATA(chp);
	len = chp->cmsg_len - (cp - (char *)chp);
	if (len < 1) {
		fprintf(stderr, "%s cmsg_len %d too small\n", __func__,
			chp->cmsg_len);
		return -1;
	}
	rlen = strlen(cp);
	if (rlen != len && rlen != len - 1) {
		fprintf(stderr, "%s len %d not strlen %d \"%s\"\n", __func__,
			len, rlen, cp);
		return -1;
	}
	if (rlen >= smacklen) {
		fprintf(stderr, "%s len %d too small, smack is %d\n", __func__,
			smacklen, rlen);
		return -1;
	}
	if (chp->cmsg_len >= smacklen) {
		fprintf(stderr, "%s len %d too small, needed %d\n", __func__,
			smacklen, chp->cmsg_len);
		return -1;
	}
	strcpy(smack, cp);
	return 0;
}
