/*
 * Copyright (C) 2012 Casey Schaufler <casey@schaufler-ca.com>
 * Copyright (C) Intel Corporation.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <linux/ip.h>
#include <linux/udp.h>
#include "smacktools.h"

#define BUFFER_SIZE 1500
#define RBS (BUFFER_SIZE - ((sizeof (struct iphdr) + sizeof (struct udphdr))))

#define LOCAL_PORT	8001
#define REMOTE_PORT	8000

int
main(int argc, char *argv[])
{
	int sock;
	struct sockaddr_in sin;
	unsigned short local_port = LOCAL_PORT;
	unsigned short remote_port = REMOTE_PORT;
	char buffer[BUFFER_SIZE];
	char rbuffer[BUFFER_SIZE];
	char *remote_ip_str = THIS_HOST;
	char *cp;
	int i;
	int reply = 0;
	int nap = 1;
	int count = 0;
	int number = -1;

	for (i = 1; i < argc; i++) {
		if (strcmp(argv[i], "-c") == 0 ||
		    strcmp(argv[i], "--count") == 0)
			count = 1;
		else if (strcmp(argv[i], "-h") == 0 ||
		    strcmp(argv[i], "--host") == 0)
			remote_ip_str = argv[++i];
		else if (strcmp(argv[i], "-p") == 0 ||
			 strcmp(argv[i], "--port") == 0)
			remote_port = atoi(argv[++i]);
		else if (strcmp(argv[i], "-s") == 0 ||
			 strcmp(argv[i], "--sleep") == 0)
			nap = atoi(argv[++i]);
		else if (strcmp(argv[i], "-n") == 0 ||
			 strcmp(argv[i], "--number") == 0)
			number = atoi(argv[++i]);
		else if (strcmp(argv[i], "-r") == 0 ||
		    strcmp(argv[i], "--reply") == 0) {
			reply = 1;
		}
		else
			fprintf(stderr, "arg \"%s\" ignored\n", argv[i]);
	}
	 
	if ((sock = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0) { 
		perror("socket");
		exit(1);
	}

	bzero((char *)&sin, sizeof(sin));
	sin.sin_family = AF_INET;
	sin.sin_port = htons(local_port);

	for (i = 0; i < 1000; i++) {
		if ((bind(sock, (struct sockaddr *)&sin, sizeof(sin))) >= 0)
			break;
		sin.sin_port = htons(local_port++);
	}

	if (i >= 1000) {
		fprintf(stderr, "failed 1000 binds\n");
		exit(1);
	}

	bzero((char *)&sin, sizeof(sin));
	sin.sin_family = AF_INET;
	sin.sin_port = htons(remote_port);
	sin.sin_addr.s_addr = inet_addr(remote_ip_str);

	if (connect(sock, (struct sockaddr *)&sin, sizeof(sin)) < 0) {
		perror("connect");
		exit(1);
	}
  
	if ((i = smack_self(buffer)) < 0) {
		fprintf(stderr, "Cannot read process Smack label.\n");
		exit(1);
	}
	cp = buffer + i;

	for (; number != 0; number--) {
		if (count != 0)
			sprintf(cp, " %d", count++);
		if (send(sock, buffer, strlen(buffer) + 1, 0) < 0)
			perror("send");
		if (reply != 0) {
			i = read(sock, rbuffer, RBS);
			printf("Reply: \"%s\", %d\n", rbuffer, i);
		}

		if (nap > 0)
			sleep(nap);
	}

	close(sock);

	return 0;
}
