/*
 * Copyright (C) 2012 Casey Schaufler <casey@schaufler-ca.com>
 * Copyright (C) Intel Corporation.
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <linux/ip.h>
#include <linux/udp.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#define THIS_HOST	"127.0.0.1"
#define IP_PASSSEC	18
#define SMACK_LABEL	256
/*
 * This are found in linux/socket.h but the code here
 * needs sys/socket.h and they conflict.
 */
#ifndef SCM_SECURITY
#define SCM_SECURITY	0x03	/* rw: security label */
#endif

static inline int smack_self(char *smack)
{
	int fd;
	int i;
	char *cp;

	fd = open("/proc/self/attr/smack/current", O_RDONLY);
	if (fd < 0)
		fd = open("/proc/self/attr/smack.current", O_RDONLY);
	if (fd < 0)
		fd = open("/proc/self/attr/current", O_RDONLY);
	if (fd < 0)
		return -1;
	i = read(fd, smack, SMACK_LABEL);
	if (i >= 0)
		smack[i] = '\0';
	if ((cp = strchr(smack, '\n')) != NULL)
		*cp = '\0';

	close(fd);
	return i;
}

static inline int smack_get_peer(int sock, char *peer, char *rawpeer)
{
	char raw[SMACK_LABEL * 2];
	int rawlen = sizeof(raw);

	if (getsockopt(sock, SOL_SOCKET, SO_PEERSEC, raw, &rawlen) < 0) {
		perror("getsockopt SO_PEERSEC");
		return -1;
	}

	if (rawpeer != NULL)
		strcpy(rawpeer, raw);
	if (sscanf(raw, "smack='%[^']'", peer) != 1)
		strcpy(peer, raw);

	return 0;
}
