/*
 *   Smack user functions
 *
 * Copyright 2007 Casey Schaufler <casey@schaufler-ca.com>
 */
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <linux/xattr.h>
#include "smacktools.h"

#define SMACKUSERFILE	"/etc/smack/user"
#define SMACKFILELINE	4096

int smack_user_default(const char *user, char *result, int len)
{
	char line[SMACKFILELINE];
	char ruser[SMACKFILELINE];
	char rsmack[SMACKFILELINE];
	FILE *fp;

	fp = fopen(SMACKUSERFILE, "r");
	if (fp == NULL)
		return -1;

	while (fgets(line, SMACKFILELINE, fp) != NULL) {
		if (line[0] == '#' || line[0] == '\n')
			continue;
		if (sscanf(line, "%s %s", ruser, rsmack) != 2)
			continue;
		if (strcmp(ruser, user) != 0)
			continue;
		if (strlen(rsmack) >= len)
			return -1;
		strcpy(result, rsmack);
		return 0;
	}
	return -1;
}

int smack_user_allowed(const char *user, const char *smack)
{
	char line[SMACKFILELINE];
	char *ruser;
	char *rsmack;
	FILE *fp;
	int rc = 1;

	fp = fopen(SMACKUSERFILE, "r");
	if (fp == NULL)
		return -1;

	while (rc == 1 && fgets(line, SMACKFILELINE, fp) != NULL) {
		if (line[0] == '#' || line[0] == '\n')
			continue;

		ruser = strtok(line, " \t\n");
		if (ruser == NULL)
			continue;

		if (strcmp(ruser, user) != 0)
			continue;

		while (rc == 1 && (rsmack = strtok(NULL, " \t\n")) != NULL) {
			if (strcmp(rsmack, smack) == 0 ||
			    strcmp(rsmack, "+") == 0)
				rc = 0;
		}
	}
	fclose(fp);
	return (rc == 1) ? -1 : rc;
}

int smack_user_add(const char *user, const char *smack)
{
	char line[SMACKFILELINE];
	char *ruser;
	FILE *fp;
	int rc = 0;

	fp = fopen(SMACKUSERFILE, "a+");
	if (fp == NULL)
		return -1;

	while (fgets(line, SMACKFILELINE, fp) != NULL) {
		if (line[0] == '#' || line[0] == '\n')
			continue;

		ruser = strtok(line, " \t\n");
		if (ruser == NULL)
			continue;

		if (strcmp(ruser, user) == 0) {
			rc = -1;
			break;
		}
	}

	if (rc == 0)
		fprintf(fp, "%s %s\n", user, smack);

	fclose(fp);
	return rc;
}
