/*
 * testaccess - See if /smack/access and /smack/access2 work
 *
 * Copyright (C) 2007 Casey Schaufler <casey@schaufler-ca.com>
 * Copyright (C) 2011 Intel Corporation.
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, version 2.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public
 *	License along with this program; if not, write to the Free Software
 *	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 *	02110-1301 USA
 *
 * Authors:
 *      Casey Schaufler <casey@schaufler-ca.com>
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <unistd.h>
#include <fcntl.h>
#include <getopt.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

char line[1024];

int writenread(int fd, char *sp, char *op, char *ap, char *fmt, int *err)
{
	char back[2];
	int rc;

	back[0] = '\0';
	back[1] = '\0';
	*err = 0;

	sprintf(line, fmt, sp, op, ap);
	if ((rc = write(fd, line, strlen(line))) < 0) {
		*err = errno;
		return -1;
	}

	if ((rc = read(fd, back, 2)) < 0) {
		*err = errno;
		return -2;
	}

	if (back[0] == '1' && back[1] == '\0')
		return 1;
	if (back[0] == '0' && back[1] == '\0')
		return 0;
	fprintf(stderr, "Read returned '%c'%c', not \"1\" or \"0\".\n",
		back[0], back[1]);
	return -3;
}

int
main(int argc, char *argv[])
{
	int rc;
	int rc2;
	int fd;
	int fd2;
	int er;
	int er2;
	int ec = 0;


	if (strcmp("access", argv[1]) == 0) {
		fd = open("/sys/fs/smackfs/access", O_RDWR);
		if (fd < 0)
			fd = open("/smack/access", O_RDWR);
		if (fd < 0) {
			perror("opening /smack/access");
			exit(5);
		}
		rc = writenread(fd, argv[2], argv[3], argv[4],
			"%-23s %-23s %-5s", &er);
	} else if (strcmp("access2", argv[1]) == 0) {
		fd = open("/sys/fs/smackfs/access2", O_RDWR);
		if (fd < 0)
			fd = open("/smack/access2", O_RDWR);
		if (fd < 0) {
			perror("opening /smack/access2");
			exit(5);
		}
		rc = writenread(fd, argv[2], argv[3], argv[4],
			"%s %s %s", &er2);
	} else {
		fprintf(stderr, "%s unrecognized\n", argv[1]);
		exit(5);
	}

	switch (rc) {
	case 0:
		fprintf(stdout, "denied\n");
		exit(1);
		break;
	case 1:
		fprintf(stdout, "allowed\n");
		exit(0);
		break;
	case -1:
		fprintf(stdout, "unwriteable\n");
		exit(2);
		break;
	case -2:
		fprintf(stdout, "unreadable\n");
		exit(3);
		break;
	case -3:
		fprintf(stdout, "mangled\n");
		exit(4);
		break;
	default:
		fprintf(stdout, "impossible\n");
		exit(5);
		break;
	}

	exit(0);
}
