#! /bin/sh
#
#

attr -q -S -s SMACK64 -V '*' /dev/null

RUNDIR="."
TESTDIR=/tmp/load-self-test
USERID=`/usr/bin/id -u`

if [ "$USERID" != "0" ]
then
	echo User ID $USERID is not root.
	exit 1
fi

NOTROOT=`tail -1 /etc/passwd | sed -e 's/:.*//'`
echo Using '"'$NOTROOT'"' as a test user.

if [ -f /proc/self/attr/smack.current ]
then
	CURRENT="/proc/self/attr/smack.current"
else
	CURRENT="/proc/self/attr/current"
fi

if [ -f /sys/fs/smackfs/load ]
then
	LOAD=/sys/fs/smackfs/load
else
	LOAD=/smack/load
fi

RUNAS=`cat $CURRENT`

rm -rf $TESTDIR

#
# Rule setting tests
#
#          1         2         3         4         5         6
# 1234567890123456789012345678901234567890123456789012345678901234567890
# subject-label.......... object-label........... rwxat
#

#                 1         2         3
#        123456789012345678901234567890
SUBJECT="Snap                   "
SUBNAME="Snap"

#                1         2         3
#       123456789012345678901234567890
OBJECT="Crackle                "
OBNAME="Crackle"

OLDRULE=`grep '^'"$SUBNAME"' '"$OBNAME" $LOAD`

echo -n "$SUBJECT"' '"$OBJECT"' ''rwxa' > $LOAD
RULE=`grep '^'"$SUBNAME"' '"$OBNAME" $LOAD`
if [ "x""$RULE" != "x""$SUBNAME"' '"$OBNAME"' rwxa' ]
then
	echo Setting rule without '"'t'"' unexpectedly '"'"$RULE"'"'.
	exit 1
fi

#
# Verify correct restriction
#

mkdir $TESTDIR
chmod 777 $TESTDIR
attr -q -S -s SMACK64 -V $OBNAME $TESTDIR
echo -n "$SUBJECT"' '"$OBJECT"' ''rwxa' > $LOAD

TESTFILE="$TESTDIR"/file

echo "$SUBNAME" > $CURRENT
su $NOTROOT -c "echo 'Test A' > $TESTFILE"
attr -q -S -s SMACK64 -V $OBNAME $TESTFILE
su $NOTROOT -c "$RUNDIR""/testloadself --file $TESTFILE"
su $NOTROOT -c "$RUNDIR""/testloadself --load-self2 --file $TESTFILE"
echo "$RUNAS" > $CURRENT

rm -rf $TESTDIR
#
# Restore old label rule.
#
if [ "x""$OLDRULE" != "x" ]
then
	RAWOLDMODE=`echo $OLDRULE | sed -e 's/.* //'`

	I=`echo $RAWOLDMODE | grep 'r'`
	if [ "x""$I" = "x" ]
	then
		OLDMODE="-"
	else
		OLDMODE="r"
	fi
	I=`echo $RAWOLDMODE | grep 'w'`
	if [ "x""$I" = "x" ]
	then
		OLDMODE="$OLDMODE""-"
	else
		OLDMODE="$OLDMODE""w"
	fi
	I=`echo $RAWOLDMODE | grep 'x'`
	if [ "x""$I" = "x" ]
	then
		OLDMODE="$OLDMODE""-"
	else
		OLDMODE="$OLDMODE""x"
	fi
	I=`echo $RAWOLDMODE | grep 'a'`
	if [ "x""$I" = "x" ]
	then
		OLDMODE="$OLDMODE""-"
	else
		OLDMODE="$OLDMODE""a"
	fi
	I=`echo $RAWOLDMODE | grep 't'`
	if [ "x""$I" = "x" ]
	then
		OLDMODE="$OLDMODE""-"
	else
		OLDMODE="$OLDMODE""t"
	fi

	echo -n "$SUBJECT"' '"$OBJECT"' '"$OLDMODE" > $LOAD
	# echo Rule reset to '"'"$SUBNAME"' '"$OBNAME"' '"$OLDMODE"'"'
else
	echo -n "$SUBJECT"' '"$OBJECT"' ''-----' > $LOAD
	# echo Rule reset to '"'"$SUBNAME"' '"$OBNAME"' '"-----"'"'
fi

echo "$RUNAS" > $CURRENT
rm -rf $TESTDIR

exit 0
